@@ -129,7 +129,7 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
 #define DocumentMedia  "%%DocumentMedia:"
 #define PageBoundingBox  "%%PageBoundingBox:"
 #define PostscriptLevel  "%!PS-"
-#define RenderPostscriptText  "  Rendering postscript...  "
+#define RenderPostscriptText  "[%s] Rendering postscript..."
 
   char
     density[MaxTextExtent],
@@ -214,7 +214,7 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
     {
       (void) strcpy(density,PSDensityGeometry);
       count=GetMagickDimension(density,&image->x_resolution,
-        &image->y_resolution);
+        &image->y_resolution,NULL,NULL);
       if (count != 2)
         image->y_resolution=image->x_resolution;
     }
@@ -331,7 +331,6 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
   if (image_info->subrange != 0)
     FormatString(options,"-dFirstPage=%lu -dLastPage=%lu",
       image_info->subimage+1,image_info->subimage+image_info->subrange);
-  FormatString(options+strlen(options)," -g%s",geometry);
   (void) strlcpy(filename,image_info->filename,MaxTextExtent);
   if (image_info->temporary)
     (void) LiberateTemporaryFile((char *) image_info->filename);
@@ -341,9 +340,10 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
       ThrowReaderTemporaryFileException(image_info->filename);
     }
   FormatString(command,delegate_info->commands,antialias,
-    antialias,density,options,image_info->filename,
+    antialias,geometry,density,options,image_info->filename,
     postscript_filename);
-  (void) MagickMonitor(RenderPostscriptText,0,8,&image->exception);
+  (void) MagickMonitorFormatted(0,8,&image->exception,RenderPostscriptText,
+                                image->filename);
   status=InvokePostscriptDelegate(image_info->verbose,command);
   if (!IsAccessibleAndNotEmpty(image_info->filename))
     {
@@ -362,7 +362,8 @@ static Image *ReadEPTImage(const ImageInfo *image_info,
                             "Returned from Postscript delegate (status=%d) ...", status);
     }
   (void) LiberateTemporaryFile(postscript_filename);
-  (void) MagickMonitor(RenderPostscriptText,7,8,&image->exception);
+  (void) MagickMonitorFormatted(7,8,&image->exception,RenderPostscriptText,
+                                image->filename);
   if (!IsAccessibleAndNotEmpty(image_info->filename))
     {
       /*
@@ -430,9 +431,9 @@ ModuleExport void RegisterEPTImage(void)
   entry->magick=(MagickHandler) IsEPT;
   entry->adjoin=False;
   entry->blob_support=False;
-  entry->description=
-    AcquireString("Adobe Encapsulated PostScript with MS-DOS TIFF preview");
-  entry->module=AcquireString("EPT");
+  entry->description="Adobe Encapsulated PostScript with MS-DOS TIFF preview";
+  entry->module="EPT";
+  entry->coder_class=PrimaryCoderClass;
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("EPT2");
@@ -441,9 +442,9 @@ ModuleExport void RegisterEPTImage(void)
   entry->magick=(MagickHandler) IsEPT;
   entry->adjoin=False;
   entry->blob_support=False;
-  entry->description=
-    AcquireString("Adobe Level II Encapsulated PostScript with MS-DOS TIFF preview");
-  entry->module=AcquireString("EPT");
+  entry->description="Adobe Level II Encapsulated PostScript with MS-DOS TIFF preview";
+  entry->module="EPT";
+  entry->coder_class=PrimaryCoderClass;
   (void) RegisterMagickInfo(entry);
 
   entry=SetMagickInfo("EPT3");
@@ -452,9 +453,9 @@ ModuleExport void RegisterEPTImage(void)
   entry->magick=(MagickHandler) IsEPT;
   entry->adjoin=False;
   entry->blob_support=False;
-  entry->description=
-    AcquireString("Adobe Level III Encapsulated PostScript with MS-DOS TIFF preview");
-  entry->module=AcquireString("EPT");
+  entry->description="Adobe Level III Encapsulated PostScript with MS-DOS TIFF preview";
+  entry->module="EPT";
+  entry->coder_class=PrimaryCoderClass;
   (void) RegisterMagickInfo(entry);
 }
 
