@@ -406,7 +406,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q=SetImagePixels(image,0,(long) real,image->columns,1);
           if (q == (PixelPacket *) NULL)
             break;
-          indexes=GetIndexes(image);
+          indexes=AccessMutableIndexes(image);
           for (x=0; x < (long) image->columns; x++)
             {
               if ((tga_info.image_type == TGARLEColormap) ||
@@ -530,7 +530,8 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
           image->is_grayscale=is_grayscale;
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(LoadImageText,y,image->rows,exception))
+              if (!MagickMonitorFormatted(y,image->rows,exception,
+                                          LoadImageText,image->filename))
                 break;
         }
       if (EOFBlob(image))
@@ -566,7 +567,9 @@ static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               return((Image *) NULL);
             }
           image=SyncNextImageInList(image);
-          if (!MagickMonitor(LoadImagesText,TellBlob(image),GetBlobSize(image),exception))
+          if (!MagickMonitorFormatted(TellBlob(image),GetBlobSize(image),
+                                      exception,LoadImagesText,
+                                      image->filename))
             break;
         }
     } while (status == True);
@@ -607,26 +610,29 @@ ModuleExport void RegisterTGAImage(void)
   entry=SetMagickInfo("ICB");
   entry->decoder=(DecoderHandler) ReadTGAImage;
   entry->encoder=(EncoderHandler) WriteTGAImage;
-  entry->description=AcquireString("Truevision Targa image");
-  entry->module=AcquireString("TGA");
+  entry->description="Truevision Targa image";
+  entry->module="TGA";
   (void) RegisterMagickInfo(entry);
+
   entry=SetMagickInfo("TGA");
   entry->decoder=(DecoderHandler) ReadTGAImage;
   entry->encoder=(EncoderHandler) WriteTGAImage;
-  entry->description=AcquireString("Truevision Targa image");
-  entry->module=AcquireString("TGA");
+  entry->description="Truevision Targa image";
+  entry->module="TGA";
   (void) RegisterMagickInfo(entry);
+
   entry=SetMagickInfo("VDA");
   entry->decoder=(DecoderHandler) ReadTGAImage;
   entry->encoder=(EncoderHandler) WriteTGAImage;
-  entry->description=AcquireString("Truevision Targa image");
-  entry->module=AcquireString("TGA");
+  entry->description="Truevision Targa image";
+  entry->module="TGA";
   (void) RegisterMagickInfo(entry);
+
   entry=SetMagickInfo("VST");
   entry->decoder=(DecoderHandler) ReadTGAImage;
   entry->encoder=(EncoderHandler) WriteTGAImage;
-  entry->description=AcquireString("Truevision Targa image");
-  entry->module=AcquireString("TGA");
+  entry->description="Truevision Targa image";
+  entry->module="TGA";
   (void) RegisterMagickInfo(entry);
 }
 
@@ -733,7 +739,7 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
   register const PixelPacket
     *p;
 
-  register IndexPacket
+  register const IndexPacket
     *indexes;
 
   register long
@@ -934,7 +940,7 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
           if (p == (const PixelPacket *) NULL)
             break;
           q=targa_pixels;
-          indexes=GetIndexes(image);
+          indexes=AccessImmutableIndexes(image);
           for (x=0; x < (long) image->columns; x++)
             {
               if (targa_info.image_type == TargaColormap)
@@ -976,14 +982,17 @@ static unsigned int WriteTGAImage(const ImageInfo *image_info,Image *image)
           (void) WriteBlob(image,q-targa_pixels,(char *) targa_pixels);
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
-              if (!MagickMonitor(SaveImageText,y,image->rows,&image->exception))
+              if (!MagickMonitorFormatted(y,image->rows,&image->exception,
+                                          SaveImageText,image->filename))
                 break;
         }
       MagickFreeMemory(targa_pixels);
       if (image->next == (Image *) NULL)
         break;
       image=SyncNextImageInList(image);
-      if (!MagickMonitor(SaveImagesText,scene++,GetImageListLength(image),&image->exception))
+      if (!MagickMonitorFormatted(scene++,GetImageListLength(image),
+                                  &image->exception,SaveImagesText,
+                                  image->filename))
         break;
     } while (image_info->adjoin);
   if (image_info->adjoin)
