@@ -34,55 +34,43 @@
 */
 #include "magick/studio.h"
 #include "magick/blob.h"
-#include "magick/cache.h"
+#include "magick/pixel_cache.h"
 #include "magick/color.h"
 #include "magick/error.h"
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/utility.h"
-
-#if !defined(SWORD)
-#  define SWORD unsigned int
-#endif
-#if !defined(SDWORD)
-#  define SDWORD unsigned long
-#endif
-#if !defined(WORD)
-#  define WORD  int
-#endif
-#if !defined(BYTE)
-#  define BYTE  unsigned char
-#endif
+
 
 typedef struct
 {
   char Name[20];
-  SWORD Rows;
-  SWORD Cols;
-  SWORD TypSou;			/* 0-binary, 1-8 bitu, 2-8 bits+PAL, 3-4 bits,
+  magick_uint16_t Rows;
+  magick_uint16_t Cols;
+  magick_uint16_t TypSou;	/* 0-binary, 1-8 bitu, 2-8 bits+PAL, 3-4 bits,
 				   4-4 bits+PAL, 5-24 bits, 6-16 bits, 7-32
 				   bits */
-  SDWORD Zoom;
-  SWORD Version;
-  SWORD Komprese;		/* 0 - uncompressed (from release 1) */
-  SWORD Stav;
+  magick_uint32_t Zoom;
+  magick_uint16_t Version;
+  magick_uint16_t Komprese;		/* 0 - uncompressed (from release 1) */
+  magick_uint16_t Stav;
   double xRasMin;
   double yRasMin;
   double xRasMax;
   double yRasMax;
   double Scale;			/* from release 2 */
-  WORD TileWidth;		/* tile width in pixels */
-  WORD TileHeight;		/* tile height in pixels */
-  SDWORD TileOffsets;		/* offset to array of longints that contains
+  magick_uint16_t TileWidth;	/* tile width in pixels */
+  magick_uint16_t TileHeight;	/* tile height in pixels */
+  magick_uint32_t TileOffsets;	/* offset to array of longints that contains
 				   adresses of tiles in the raster (adreses
 				   are counted from 0) */
-  SDWORD TileByteCounts;	/* offset to array of words, that contain amount of bytes stored in
+  magick_uint32_t TileByteCounts;/* offset to array of words, that contain amount of bytes stored in
 				   tiles. The tile size might vary depending on
 				   the value TileCompression */
-  BYTE TileCompression;		/* 0 - uncompressed, 1 - variant TIFF
+  magick_uint8_t TileCompression;/* 0 - uncompressed, 1 - variant TIFF
 				   Packbits, 2 - CCITT G3 */
 
-  BYTE Dummy[423];
+  magick_uint8_t Dummy[423];
 } RasHeader;
 
 /*
@@ -118,12 +106,12 @@ static void InsertRow(int depth, unsigned char *p, long y, Image * image, unsign
 	q = SetImagePixels(image, 0, y, image->columns, 1);
 	if (q == (PixelPacket *) NULL)
 	  break;
-	indexes = GetIndexes(image);
+	indexes = AccessMutableIndexes(image);
 	for (x = 0; x < ((long) image->columns - 7); x += 8)
           {
             for (bit = 0; bit < 8; bit++)
               {
-                index = ((*p) & (0x80 >> bit) ? 0x01 : 0x00);
+                index = ((*p) & (0x80U >> bit) ? 0x01U : 0x00U);
                 indexes[x + bit] = MEZ[index];
                 *q++ = image->colormap[index];
               }
@@ -151,7 +139,7 @@ static void InsertRow(int depth, unsigned char *p, long y, Image * image, unsign
 	q = SetImagePixels(image, 0, y, image->columns, 1);
 	if (q == (PixelPacket *) NULL)
 	  break;
-	indexes = GetIndexes(image);
+	indexes = AccessMutableIndexes(image);
 	for (x = 0; x < ((long) image->columns - 1); x += 2)
           {
             index = (IndexPacket) ((*p >> 6) & 0x3);
@@ -207,7 +195,7 @@ static void InsertRow(int depth, unsigned char *p, long y, Image * image, unsign
 	q = SetImagePixels(image, 0, y, image->columns, 1);
 	if (q == (PixelPacket *) NULL)
 	  break;
-	indexes = GetIndexes(image);
+	indexes = AccessMutableIndexes(image);
 	for (x = 0; x < ((long) image->columns - 1); x += 2)
           {
             index = (IndexPacket) ((*p >> 4) & 0xf);
@@ -240,7 +228,7 @@ static void InsertRow(int depth, unsigned char *p, long y, Image * image, unsign
 	q = SetImagePixels(image, 0, y, image->columns, 1);
 	if (q == (PixelPacket *) NULL)
 	  break;
-	indexes = GetIndexes(image);
+	indexes = AccessMutableIndexes(image);
 
 	for (x = 0; x < (long) image->columns; x++)
           {
@@ -260,47 +248,6 @@ static void InsertRow(int depth, unsigned char *p, long y, Image * image, unsign
     }
 }
 
-static double ReadBlobLSBdouble(Image * image)
-{
-  typedef union
-  {
-    double d;
-    char chars[8];
-  } dbl;
-
-  static unsigned long
-    lsb_first = 1;
-
-  dbl
-    buffer;
-
-  char
-    c;
-
-  assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-
-  if (ReadBlob(image, 8, (unsigned char *) &buffer) == 0)
-    return (0.0);
-
-  if (*(char *) &lsb_first == 1)
-    return (buffer.d);
-
-  c = buffer.chars[0];
-  buffer.chars[0] = buffer.chars[7];
-  buffer.chars[7] = c;
-  c = buffer.chars[1];
-  buffer.chars[1] = buffer.chars[6];
-  buffer.chars[6] = c;
-  c = buffer.chars[2];
-  buffer.chars[2] = buffer.chars[5];
-  buffer.chars[5] = c;
-  c = buffer.chars[3];
-  buffer.chars[3] = buffer.chars[4];
-  buffer.chars[4] = c;
-  return (buffer.d);
-}
-
 /* This function reads one block of unsigned shortS */
 static void ReadBlobWordLSB(Image * image, size_t len, unsigned short *data)
 {
@@ -368,8 +315,7 @@ static Image *ReadTOPOLImage(const ImageInfo * image_info, ExceptionInfo * excep
   RasHeader
     Header;
 
-  /* paletteRAS
-     Pal; */
+  int logging;
 
   int
     depth,
@@ -395,6 +341,9 @@ static Image *ReadTOPOLImage(const ImageInfo * image_info, ExceptionInfo * excep
   assert(image_info->signature == MagickSignature);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
+
+  logging = LogMagickEvent(CoderEvent,GetMagickModule(),"enter"); 
+
   image = AllocateImage(image_info);
   status = OpenBlob(image_info, image, ReadBinaryBlobMode, exception);
   if (status == False)
@@ -403,8 +352,8 @@ static Image *ReadTOPOLImage(const ImageInfo * image_info, ExceptionInfo * excep
   /*
     Read TopoL RAS header.
   */
-  memset(&Header, 0, sizeof(Header));
-  ReadBlob(image, 20, &Header.Name);
+  (void) memset(&Header, 0, sizeof(Header));
+  (void) ReadBlob(image, 20, &Header.Name);
   Header.Rows = ReadBlobLSBShort(image);
   Header.Cols = ReadBlobLSBShort(image);
   Header.TypSou = ReadBlobLSBShort(image);
@@ -414,13 +363,13 @@ static Image *ReadTOPOLImage(const ImageInfo * image_info, ExceptionInfo * excep
     {
       Header.Komprese = ReadBlobLSBShort(image);
       Header.Stav = ReadBlobLSBShort(image);
-      Header.xRasMin = ReadBlobLSBdouble(image);
-      Header.yRasMin = ReadBlobLSBdouble(image);
-      Header.xRasMax = ReadBlobLSBdouble(image);
-      Header.yRasMax = ReadBlobLSBdouble(image);
+      Header.xRasMin = ReadBlobLSBDouble(image);
+      Header.yRasMin = ReadBlobLSBDouble(image);
+      Header.xRasMax = ReadBlobLSBDouble(image);
+      Header.yRasMax = ReadBlobLSBDouble(image);
       if (Header.Version >= 2)	/* from release 2 */
         {
-          Header.Scale = ReadBlobLSBdouble(image);
+          Header.Scale = ReadBlobLSBDouble(image);
           Header.TileWidth = ReadBlobLSBShort(image);
           Header.TileHeight = ReadBlobLSBShort(image);
           Header.TileOffsets = ReadBlobLSBLong(image);
@@ -430,7 +379,7 @@ static Image *ReadTOPOLImage(const ImageInfo * image_info, ExceptionInfo * excep
         }
     }
 
-  for (i = 0; i < sizeof(Header.Name); i++)
+  for (i = 0; i < (long) sizeof(Header.Name); i++)
     {
       if (Header.Name[i] < ' ')
         TOPOL_KO:ThrowReaderException(CorruptImageError,ImproperImageHeader,
@@ -510,9 +459,9 @@ static Image *ReadTOPOLImage(const ImageInfo * image_info, ExceptionInfo * excep
       (void) strcpy(clone_info->filename+i,".mez");
       if((clone_info->file=fopen(clone_info->filename,"rb"))==NULL)
         {
-        DestroyImageInfo(clone_info);
-        clone_info=NULL;
-        goto NoMEZ;            
+          DestroyImageInfo(clone_info);
+          clone_info=NULL;
+          goto NoMEZ;            
         }
     }
 
@@ -521,15 +470,16 @@ static Image *ReadTOPOLImage(const ImageInfo * image_info, ExceptionInfo * excep
   if (status == False) goto NoMEZ; 
 
   ldblk=(long) GetBlobSize(palette);
-  if(ldblk>sizeof(MEZ)) ldblk=sizeof(MEZ);
-  ReadBlob(palette, ldblk, MEZ); 
+  if ( ldblk > (long) sizeof(MEZ))
+    ldblk=sizeof(MEZ);
+  (void) ReadBlob(palette, ldblk, MEZ); 
             
-NoMEZ:		/*Clean up palette and clone_info*/
+ NoMEZ:		/*Clean up palette and clone_info*/
   if (palette != NULL) {DestroyImage(palette);palette=NULL;}
   if (clone_info != NULL) 
     {
-    DestroyImageInfo(clone_info);
-    clone_info=NULL;
+      DestroyImageInfo(clone_info);
+      clone_info=NULL;
     }
 
   /* ----- Do something with palette ----- */
@@ -571,7 +521,7 @@ NoMEZ:		/*Clean up palette and clone_info*/
   status=OpenBlob(clone_info,palette,ReadBinaryBlobMode,exception);
   if (status == False)
     {
-ErasePalette:     
+    ErasePalette:     
       DestroyImage(palette);
       palette=NULL;
       goto NoPalette;
@@ -580,36 +530,36 @@ ErasePalette:
      
   if(palette!=NULL)
     {
-    ldblk=ReadBlobByte(palette);		/*size of palette*/
-    if(ldblk==EOF) goto ErasePalette;
-    image->colors=ldblk+1;    
-    if (!AllocateImageColormap(image, image->colors)) goto NoMemory;
+      ldblk=ReadBlobByte(palette);		/*size of palette*/
+      if(ldblk==EOF) goto ErasePalette;
+      image->colors=ldblk+1;    
+      if (!AllocateImageColormap(image, image->colors)) goto NoMemory;
     
-    for(i=0;i<=ldblk;i++)
+      for(i=0;i<=ldblk;i++)
         {     
-        j=ReadBlobByte(palette);	/* Flag */
-        if(j==EOF) break;		/* unexpected end of file */
-	if(j<=ldblk)
-	  {
-          image->colormap[j].red=ScaleCharToQuantum(ReadBlobByte(palette));
-          image->colormap[j].green=ScaleCharToQuantum(ReadBlobByte(palette));
-          image->colormap[j].blue=ScaleCharToQuantum(ReadBlobByte(palette));       
-	  }
-	else
-	  {
-	  SeekBlob(palette, 3, SEEK_CUR);
-	  fprintf(stderr,"TopoL: Wrong index inside palette %d!",(int)j);
-	  } 
+          j=ReadBlobByte(palette);	/* Flag */
+          if(j==EOF) break;		/* unexpected end of file */
+          if(j<=ldblk)
+            {
+              image->colormap[j].red = ScaleCharToQuantum(ReadBlobByte(palette));
+              image->colormap[j].green = ScaleCharToQuantum(ReadBlobByte(palette));
+              image->colormap[j].blue = ScaleCharToQuantum(ReadBlobByte(palette));
+            }
+          else
+            {
+              (void) SeekBlob(palette, 3, SEEK_CUR);
+              (void) fprintf(stderr,"TopoL: Wrong index inside palette %d!",(int)j);
+            } 
         }
     }
 
 
-NoPalette:
+ NoPalette:
   if (palette == NULL && image->colors != 0)
     {
       if (!AllocateImageColormap(image, image->colors))
         {
-NoMemory:
+        NoMemory:
           ThrowReaderException(ResourceLimitError, MemoryAllocationFailed, image);
         }
 
@@ -623,7 +573,7 @@ NoMemory:
 
   /* ----- Load TopoL raster ----- */
   ldblk = (long) ((depth * image->columns + 7) / 8);
-  BImgBuff = (unsigned char *) AcquireMemory(ldblk);	/*Ldblk was set in the check phase */
+  BImgBuff = MagickAllocateMemory(unsigned char *,(size_t) ldblk);	/*Ldblk was set in the check phase */
   if (BImgBuff == NULL)
     goto NoMemory;
 
@@ -654,6 +604,8 @@ NoMemory:
   /* if (EOFBlob(image))
      ThrowReaderException(CorruptImageError,UnexpectedEndOfFile,image); */
   CloseBlob(image);
+
+  if (logging) (void)LogMagickEvent(CoderEvent,GetMagickModule(),"return");
   return (image);
 }
 
@@ -687,8 +639,8 @@ ModuleExport void RegisterTOPOLImage(void)
   entry = SetMagickInfo("TOPOL");
   entry->decoder = (DecoderHandler) ReadTOPOLImage;
   entry->seekable_stream = True;
-  entry->description = AcquireString("TOPOL X Image");
-  entry->module = AcquireString("TOPOL");
+  entry->description = "TOPOL X Image";
+  entry->module = "TOPOL";
   (void) RegisterMagickInfo(entry);
 }
 
