@@ -39,25 +39,58 @@
   Include declarations.
 */
 #include "magick/studio.h"
-#include "magick/cache.h"
+#include "magick/alpha_composite.h"
 #include "magick/color.h"
 #include "magick/composite.h"
 #include "magick/constitute.h"
 #include "magick/gem.h"
 #include "magick/log.h"
+#include "magick/pixel_cache.h"
 #include "magick/render.h"
 #include "magick/tempfile.h"
 #include "magick/transform.h"
 #include "magick/utility.h"
 #include "magick/xwindow.h"
 #if defined(HasTTF)
+
 #if defined(__MINGW32__)
 #  undef interface  /* Remove interface define */
 #endif
-#include "freetype/freetype.h"
-#include "freetype/ftglyph.h"
-#include "freetype/ftoutln.h"
-#endif
+
+#if defined(HAVE_FT2BUILD_H)
+  /*
+    Modern libwmf2 installs may require that <ft2build.h> be included
+    before including other libwmf headers.  Including <ft2build.h>
+    establishes definitions used by other FreeType headers.
+  */
+#  include <ft2build.h>
+#endif /* defined(HAVE_FT2BUILD_H) */
+
+#if defined(FT_FREETYPE_H)
+#  include FT_FREETYPE_H
+#else
+#  include <freetype/freetype.h>
+#endif /* defined(FT_FREETYPE_H) */
+
+#if defined(FT_GLYPH_H)
+#  include FT_GLYPH_H
+#else
+#  include <freetype/ftglyph.h>
+#endif /* defined(FT_GLYPH_H) */
+
+#if defined(FT_OUTLINE_H)
+#  include FT_OUTLINE_H
+#else
+#  include <freetype/ftoutln.h>
+#endif /* defined(FT_OUTLINE_H) */
+
+#if defined(FT_BBOX_H)
+#  include FT_BBOX_H
+#else
+#  include <freetype/ftbbox.h>
+#endif /* defined(FT_BBOX_H) */
+
+#endif /* defined(HasTTF) */
 
 /*
   Forward declarations.
@@ -122,7 +155,7 @@ static unsigned int
 %
 %
 */
-MagickExport unsigned int AnnotateImage(Image *image,const DrawInfo *draw_info)
+MagickExport MagickPassFail AnnotateImage(Image *image,const DrawInfo *draw_info)
 {
   char
     primitive[MaxTextExtent],
@@ -149,8 +182,10 @@ MagickExport unsigned int AnnotateImage(Image *image,const DrawInfo *draw_info)
     metrics;
 
   unsigned int
-    matte,
-    status;
+    matte;
+
+  MagickPassFail
+    status=MagickPass;
 
   unsigned long
     height,
@@ -164,26 +199,26 @@ MagickExport unsigned int AnnotateImage(Image *image,const DrawInfo *draw_info)
   assert(draw_info != (DrawInfo *) NULL);
   assert(draw_info->signature == MagickSignature);
   if (draw_info->text == (char *) NULL)
-    return(False);
+    return(MagickFail);
   if (*draw_info->text == '\0')
-    return(False);
+    return(MagickFail);
   text=TranslateText((ImageInfo *) NULL,image,draw_info->text);
   if (text == (char *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      "UnableToAnnotateImage");
+    ThrowBinaryException3(ResourceLimitError,MemoryAllocationFailed,
+      UnableToAnnotateImage);
   textlist=StringToList(text);
-  LiberateMemory((void **) &text);
+  MagickFreeMemory(text);
   if (textlist == (char **) NULL)
-    return(False);
+    return(MagickFail);
   length=strlen(textlist[0]);
   for (i=1; textlist[i] != (char *) NULL; i++)
     if (strlen(textlist[i]) > length)
       length=strlen(textlist[i]);
   number_lines=i;
-  text=(char *) AcquireMemory(length+MaxTextExtent);
+  text=MagickAllocateMemory(char *,length+MaxTextExtent);
   if (text == (char *) NULL)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      "UnableToAnnotateImage");
+    ThrowBinaryException3(ResourceLimitError,MemoryAllocationFailed,
+      UnableToAnnotateImage);
   SetGeometry(image,&geometry);
   if (draw_info->geometry != (char *) NULL)
     (void) GetGeometry(draw_info->geometry,&geometry.x,&geometry.y,
@@ -353,7 +388,7 @@ MagickExport unsigned int AnnotateImage(Image *image,const DrawInfo *draw_info)
       Annotate image with text.
     */
     status=RenderType(image,annotate,&offset,&metrics);
-    if (status == False)
+    if (status == MagickFail)
       break;
     if (annotate->decorate == LineThroughDecoration)
       {
@@ -369,10 +404,10 @@ MagickExport unsigned int AnnotateImage(Image *image,const DrawInfo *draw_info)
   */
   DestroyDrawInfo(clone_info);
   DestroyDrawInfo(annotate);
-  LiberateMemory((void **) &text);
+  MagickFreeMemory(text);
   for (i=0; textlist[i] != (char *) NULL; i++)
-    LiberateMemory((void **) &textlist[i]);
-  LiberateMemory((void **) &textlist);
+    MagickFreeMemory(textlist[i]);
+  MagickFreeMemory(textlist);
   return(status);
 }
 
@@ -455,11 +490,11 @@ static unsigned short *EncodeSJIS(const char *text,size_t *count)
   *count=0;
   if ((text == (char *) NULL) || (*text == '\0'))
     return((unsigned short *) NULL);
-  encoding=(unsigned short *)
-    AcquireMemory((strlen(text)+MaxTextExtent)*sizeof(unsigned short));
+  encoding=MagickAllocateMemory(unsigned short *,
+    (strlen(text)+MaxTextExtent)*sizeof(unsigned short));
   if (encoding == (unsigned short *) NULL)
-    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
-      "UnableToConvertText");
+    MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
+      UnableToConvertText);
   q=encoding;
   for (p=text; *p != '\0'; p+=length)
   {
@@ -522,11 +557,11 @@ static unsigned short *EncodeText(const char *text,size_t *count)
   *count=0;
   if ((text == (char *) NULL) || (*text == '\0'))
     return((unsigned short *) NULL);
-  encoding=(unsigned short *)
-    AcquireMemory((strlen(text)+MaxTextExtent)*sizeof(unsigned short));
+  encoding=MagickAllocateMemory(unsigned short *,
+    (strlen(text)+MaxTextExtent)*sizeof(unsigned short));
   if (encoding == (unsigned short *) NULL)
-    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
-      "UnableToConvertText");
+    MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
+      UnableToConvertText);
   q=encoding;
   for (p=text; *p != '\0'; p++)
     *q++=(unsigned char) *p;
@@ -636,11 +671,11 @@ static unsigned short *EncodeUnicode(const char *text,size_t *count)
   *count=0;
   if ((text == (char *) NULL) || (*text == '\0'))
     return((unsigned short *) NULL);
-  unicode=(unsigned short *)
-    AcquireMemory((strlen(text)+MaxTextExtent)*sizeof(unsigned short));
+  unicode=MagickAllocateMemory(unsigned short *,
+    (strlen(text)+MaxTextExtent)*sizeof(unsigned short));
   if (unicode == (unsigned short *) NULL)
-    MagickFatalError(ResourceLimitFatalError,"MemoryAllocationFailed",
-      "UnableToConvertText");
+    MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
+      UnableToConvertText);
   q=unicode;
   for (p=text; *p != '\0'; p+=length)
   {
@@ -700,7 +735,7 @@ static unsigned short *EncodeUnicode(const char *text,size_t *count)
 %
 %
 */
-MagickExport unsigned int GetTypeMetrics(Image *image,const DrawInfo *draw_info,
+MagickExport MagickPassFail GetTypeMetrics(Image *image,const DrawInfo *draw_info,
   TypeMetric *metrics)
 {
   DrawInfo
@@ -709,7 +744,7 @@ MagickExport unsigned int GetTypeMetrics(Image *image,const DrawInfo *draw_info,
   PointInfo
     offset;
 
-  unsigned int
+  MagickPassFail
     status;
 
   assert(draw_info != (DrawInfo *) NULL);
@@ -717,7 +752,7 @@ MagickExport unsigned int GetTypeMetrics(Image *image,const DrawInfo *draw_info,
   assert(draw_info->signature == MagickSignature);
   clone_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
   clone_info->render=False;
-  memset(metrics,0,sizeof(TypeMetric));
+  (void) memset(metrics,0,sizeof(TypeMetric));
   offset.x=0.0;
   offset.y=0.0;
   status=RenderType(image,clone_info,&offset,metrics);
@@ -759,7 +794,7 @@ MagickExport unsigned int GetTypeMetrics(Image *image,const DrawInfo *draw_info,
 %
 %
 */
-static unsigned int RenderType(Image *image,const DrawInfo *draw_info,
+static MagickPassFail RenderType(Image *image,const DrawInfo *draw_info,
   const PointInfo *offset,TypeMetric *metrics)
 {
   const TypeInfo
@@ -768,7 +803,7 @@ static unsigned int RenderType(Image *image,const DrawInfo *draw_info,
   DrawInfo
    *clone_info;
 
-  unsigned int
+  MagickPassFail
     status;
 
   type_info=(const TypeInfo *) NULL;
@@ -833,29 +868,6 @@ static unsigned int RenderType(Image *image,const DrawInfo *draw_info,
 %
 */
 
-static inline PixelPacket AlphaComposite(const PixelPacket *p,
-  const double alpha,const PixelPacket *q,const double beta)
-{
-  PixelPacket
-    composite;
-
-  double
-    MaxRGB_alpha,
-    MaxRGB_beta;
-
-  MaxRGB_alpha=MaxRGB-alpha;
-  MaxRGB_beta=MaxRGB-beta;
-  composite.red=(Quantum)
-    ((MaxRGB_alpha*p->red+alpha*MaxRGB_beta*q->red/MaxRGB)/MaxRGB+0.5);
-  composite.green=(Quantum)
-    ((MaxRGB_alpha*p->green+alpha*MaxRGB_beta*q->green/MaxRGB)/MaxRGB+0.5);
-  composite.blue=(Quantum)
-    ((MaxRGB_alpha*p->blue+alpha*MaxRGB_beta*q->blue/MaxRGB)/MaxRGB+0.5);
-  composite.opacity=(Quantum)
-    (MaxRGB-(MaxRGB_alpha+alpha*MaxRGB_beta/MaxRGB)+0.5);
-  return(composite);
-}
-
 #if defined(HasTTF)
 static int TraceCubicBezier(FT_Vector *p,FT_Vector *q,FT_Vector *to,
   DrawInfo *draw_info)
@@ -918,7 +930,7 @@ static int TraceQuadraticBezier(FT_Vector *control,FT_Vector *to,
   return(0);
 }
 
-static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
+static MagickPassFail RenderFreetype(Image *image,const DrawInfo *draw_info,
   const char *encoding,const PointInfo *offset,TypeMetric *metrics)
 {
   typedef struct _GlyphInfo
@@ -1009,12 +1021,15 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
   unsigned short
     *text;
 
+  glyph.image=(FT_Glyph) 0;
+  last_glyph.image=(FT_Glyph) 0;
+
   /*
     Initialize Truetype library.
   */
   status=FT_Init_FreeType(&library);
   if (status)
-    ThrowBinaryException(TypeError,"UnableToInitializeFreetypeLibrary",
+    ThrowBinaryException(TypeError,UnableToInitializeFreetypeLibrary,
       draw_info->font);
   if (*draw_info->font != '@')
     status=FT_New_Face(library,draw_info->font,0,&face);
@@ -1023,7 +1038,7 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
   if (status != 0)
     {
       (void) FT_Done_FreeType(library);
-      ThrowBinaryException(DelegateError,"UnableToReadFont",draw_info->font)
+      ThrowBinaryException(TypeError,UnableToReadFont,draw_info->font)
     }
   if (face->num_charmaps != 0)
     status=FT_Set_Charmap(face,face->charmaps[0]);
@@ -1060,7 +1075,7 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
         encoding_type=ft_encoding_wansung;
       status=FT_Select_Charmap(face,encoding_type);
       if (status != 0)
-        ThrowBinaryException(TypeError,"UnrecognizedFontEncoding",encoding);
+        ThrowBinaryException(TypeError,UnrecognizedFontEncoding,encoding);
     }
   /*
     Set text size.
@@ -1069,7 +1084,7 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
   resolution.y=72.0;
   if (draw_info->density != (char *) NULL)
     {
-      i=sscanf(draw_info->density,"%lfx%lf",&resolution.x,&resolution.y);
+      i=GetMagickDimension(draw_info->density,&resolution.x,&resolution.y,NULL,NULL);
       if (i != 2)
         resolution.y=resolution.x;
     }
@@ -1128,7 +1143,7 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
     {
       (void) FT_Done_Face(face);
       (void) FT_Done_FreeType(library);
-      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
+      ThrowBinaryException(ResourceLimitError,MemoryAllocationFailed,
         draw_info->font)
     }
   /*
@@ -1165,13 +1180,27 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
         origin.x+=kerning.x;
       }
     glyph.origin=origin;
+    glyph.image=0;
     status=FT_Load_Glyph(face,glyph.id,FT_LOAD_DEFAULT);
     if (status != False)
       continue;
     status=FT_Get_Glyph(face->glyph,&glyph.image);
     if (status != False)
       continue;
-    FT_Glyph_Get_CBox(glyph.image,0,&bounds);
+#if 0
+    /*
+      Obtain glyph's control box. Usually faster than computing the
+      exact bounding box but may be slightly larger in some
+      situations.
+    */
+    (void) FT_Glyph_Get_CBox(glyph.image,FT_GLYPH_BBOX_SUBPIXELS,&bounds);
+#else
+    /*
+      Compute exact bounding box for scaled outline. If necessary, the
+      outline Bezier arcs are walked over to extract their extrema.
+    */
+    (void) FT_Outline_Get_BBox(&((FT_OutlineGlyph) glyph.image)->outline,&bounds);
+#endif
     if ((i == 0) || (bounds.xMin < metrics->bounds.x1))
       metrics->bounds.x1=bounds.xMin;
     if ((i == 0) || (bounds.yMin < metrics->bounds.y1))
@@ -1211,6 +1240,7 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
             point.x=offset->x+bitmap->left;
             point.y=offset->y-bitmap->top;
             p=bitmap->bitmap.buffer;
+            /* FIXME: OpenMP */
             for (y=0; y < (long) bitmap->bitmap.rows; y++)
             {
               if ((ceil(point.y+y-0.5) < 0) ||
@@ -1237,7 +1267,7 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
                   opacity=((*p) < 127 ? OpaqueOpacity : TransparentOpacity);
                 fill_color=draw_info->fill;
                 if (pattern != (Image *) NULL)
-                  fill_color=AcquireOnePixel(pattern,
+                  (void) AcquireOnePixelByReference(pattern,&fill_color,
                     (long) (point.x+x-pattern->tile_info.x) % pattern->columns,
                     (long) (point.y+y-pattern->tile_info.y) % pattern->rows,
                             &image->exception);
@@ -1251,8 +1281,8 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
                     q++;
                     continue;
                   }
-                *q=AlphaComposite(&fill_color,opacity,q,
-                                  image->matte ? q->opacity : OpaqueOpacity);
+                AlphaCompositePixel(q,&fill_color,opacity,q,
+                                    image->matte ? q->opacity : OpaqueOpacity);
                 if (!active)
                   (void) SyncImagePixels(image);
                 p++;
@@ -1291,17 +1321,18 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
   /*
     Free resources.
   */
-  LiberateMemory((void **) &text);
+  MagickFreeMemory(text);
   DestroyDrawInfo(clone_info);
   (void) FT_Done_Face(face);
   (void) FT_Done_FreeType(library);
-  return(True);
+  return(MagickPass);
 }
 #else
 static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
-  const char *encoding,const PointInfo *offset,TypeMetric *metrics)
+  const char * ARGUNUSED(encoding),const PointInfo * ARGUNUSED(offset),
+  TypeMetric * ARGUNUSED(metrics))
 {
-  ThrowBinaryException(MissingDelegateError,"FreeTypeLibraryIsNotAvailable",
+  ThrowBinaryException(MissingDelegateError,FreeTypeLibraryIsNotAvailable,
     draw_info->font)
 }
 #endif
@@ -1371,7 +1402,7 @@ static char *EscapeParenthesis(const char *text)
   return(buffer);
 }
 
-static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
+static MagickPassFail RenderPostscript(Image *image,const DrawInfo *draw_info,
   const PointInfo *offset,TypeMetric *metrics)
 {
   char
@@ -1415,7 +1446,7 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
     draw_info->font : "none",draw_info->pointsize);
   file=AcquireTemporaryFileStream(filename,BinaryFileIOMode);
   if (file == (FILE *) NULL)
-    ThrowBinaryException(FileOpenError,"UnableToCreateTemporaryFile",filename);
+    ThrowBinaryException(FileOpenError,UnableToCreateTemporaryFile,filename);
   (void) fprintf(file,"%%!PS-Adobe-3.0\n");
   (void) fprintf(file,"/ReencodeType\n");
   (void) fprintf(file,"{\n");
@@ -1461,7 +1492,7 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
     (void) fprintf(file,"(%.1024s) stringwidth pop -0.5 mul -0.5 rmoveto\n",
       text);
   (void) fprintf(file,"(%.1024s) show\n",text);
-  LiberateMemory((void **) &text);
+  MagickFreeMemory(text);
   (void) fprintf(file,"showpage\n");
   (void) fclose(file);
   FormatString(geometry,"%ldx%ld+0+0!",(long) ceil(extent.x-0.5),
@@ -1474,10 +1505,10 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
   clone_info->antialias=draw_info->text_antialias;
   annotate_image=ReadImage(clone_info,&image->exception);
   if (image->exception.severity != UndefinedException)
-    MagickError(image->exception.severity,image->exception.reason,
+    MagickError2(image->exception.severity,image->exception.reason,
       image->exception.description);
   DestroyImageInfo(clone_info);
-  LiberateTemporaryFile(filename);
+  (void) LiberateTemporaryFile(filename);
   if (annotate_image == (Image *) NULL)
     return(False);
   resolution.x=72.0;
@@ -1487,7 +1518,7 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
       int
         count;
 
-      count=sscanf(draw_info->density,"%lfx%lf",&resolution.x,&resolution.y);
+      count=GetMagickDimension(draw_info->density,&resolution.x,&resolution.y,NULL,NULL);
       if (count != 2)
         resolution.y=resolution.x;
     }
@@ -1533,7 +1564,7 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
       /*
         Render fill color.
       */
-      SetImageType(annotate_image,TrueColorMatteType);
+      (void) SetImageType(annotate_image,TrueColorMatteType);
       fill_color=draw_info->fill;
       pattern=draw_info->fill_pattern;
       for (y=0; y < (long) annotate_image->rows; y++)
@@ -1544,7 +1575,7 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
         for (x=0; x < (long) annotate_image->columns; x++)
         {
           if (pattern != (Image *) NULL)
-            fill_color=AcquireOnePixel(pattern,
+            (void) AcquireOnePixelByReference(pattern,&fill_color,
               (long) (x-pattern->tile_info.x) % pattern->columns,
               (long) (y-pattern->tile_info.y) % pattern->rows,
               &image->exception);
@@ -1564,7 +1595,7 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
         metrics->descent)-0.5));
     }
   DestroyImage(annotate_image);
-  return(True);
+  return(MagickPass);
 }
 
 /*
@@ -1602,7 +1633,7 @@ static unsigned int RenderPostscript(Image *image,const DrawInfo *draw_info,
 %
 */
 #if defined(HasX11)
-static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
+static MagickPassFail RenderX11(Image *image,const DrawInfo *draw_info,
   const PointInfo *offset,TypeMetric *metrics)
 {
   static DrawInfo
@@ -1611,16 +1642,16 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
   static Display
     *display = (Display *) NULL;
 
-  static XAnnotateInfo
+  static MagickXAnnotateInfo
     annotate_info;
 
   static XFontStruct
     *font_info;
 
-  static XPixelInfo
+  static MagickXPixelInfo
     pixel;
 
-  static XResourceInfo
+  static MagickXResourceInfo
     resource_info;
 
   static XrmDatabase
@@ -1632,7 +1663,7 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
   static XVisualInfo
     *visual_info;
 
-  unsigned int
+  MagickPassFail
     status;
 
   unsigned long
@@ -1649,15 +1680,15 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
       */
       display=XOpenDisplay(draw_info->server_name);
       if (display == (Display *) NULL)
-        ThrowBinaryException(XServerError,"UnableToOpenXServer",
+        ThrowBinaryException(XServerError,UnableToOpenXServer,
           draw_info->server_name);
       /*
         Get user defaults from X resource database.
       */
-      (void) XSetErrorHandler(XError);
+      (void) XSetErrorHandler(MagickXError);
       client_name=SetClientName((char *) NULL);
-      resource_database=XGetResourceDatabase(display,client_name);
-      XGetResourceInfo(resource_database,client_name,&resource_info);
+      resource_database=MagickXGetResourceDatabase(display,client_name);
+      MagickXGetResourceInfo(resource_database,client_name,&resource_info);
       resource_info.close_server=False;
       resource_info.colormap=PrivateColormap;
       resource_info.font=AllocateString(draw_info->font);
@@ -1665,38 +1696,38 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
       resource_info.foreground_color=AllocateString("#000000000000");
       map_info=XAllocStandardColormap();
       if (map_info == (XStandardColormap *) NULL)
-        ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-          "UnableToAllocateColormap");
+        ThrowBinaryException3(ResourceLimitError,MemoryAllocationFailed,
+          UnableToAllocateColormap);
       /*
         Initialize visual info.
       */
-      visual_info=XBestVisualInfo(display,map_info,&resource_info);
+      visual_info=MagickXBestVisualInfo(display,map_info,&resource_info);
       if (visual_info == (XVisualInfo *) NULL)
-        ThrowBinaryException(XServerError,"UnableToGetVisual",
+        ThrowBinaryException(XServerError,UnableToGetVisual,
           draw_info->server_name);
       map_info->colormap=(Colormap) NULL;
       pixel.pixels=(unsigned long *) NULL;
       /*
         Initialize Standard Colormap info.
       */
-      XGetMapInfo(visual_info,XDefaultColormap(display,visual_info->screen),
+      MagickXGetMapInfo(visual_info,XDefaultColormap(display,visual_info->screen),
         map_info);
-      XGetPixelPacket(display,visual_info,map_info,&resource_info,
+      MagickXGetPixelPacket(display,visual_info,map_info,&resource_info,
         (Image *) NULL,&pixel);
       pixel.annotate_context=XDefaultGC(display,visual_info->screen);
       /*
         Initialize font info.
       */
-      font_info=XBestFont(display,&resource_info,False);
+      font_info=MagickXBestFont(display,&resource_info,False);
       if (font_info == (XFontStruct *) NULL)
-        ThrowBinaryException(XServerError,"UnableToLoadFont",draw_info->font);
+        ThrowBinaryException(XServerError,UnableToLoadFont,draw_info->font);
       if ((map_info == (XStandardColormap *) NULL) ||
           (visual_info == (XVisualInfo *) NULL) ||
           (font_info == (XFontStruct *) NULL))
         {
-          XFreeResources(display,visual_info,map_info,&pixel,font_info,
-            &resource_info,(XWindowInfo *) NULL);
-          ThrowBinaryException(XServerError,"UnableToLoadFont",
+          MagickXFreeResources(display,visual_info,map_info,&pixel,font_info,
+            &resource_info,(MagickXWindowInfo *) NULL);
+          ThrowBinaryException(XServerError,UnableToLoadFont,
             draw_info->server_name)
         }
       cache_info=(*draw_info);
@@ -1704,7 +1735,7 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
   /*
     Initialize annotate info.
   */
-  XGetAnnotateInfo(&annotate_info);
+  MagickXGetAnnotateInfo(&annotate_info);
   annotate_info.stencil=ForegroundStencil;
   if (cache_info.font != draw_info->font)
     {
@@ -1713,9 +1744,9 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
       */
       (void) XFreeFont(display,font_info);
       (void) CloneString(&resource_info.font,draw_info->font);
-      font_info=XBestFont(display,&resource_info,False);
+      font_info=MagickXBestFont(display,&resource_info,False);
       if (font_info == (XFontStruct *) NULL)
-        ThrowBinaryException(XServerError,"UnableToLoadFont",draw_info->font);
+        ThrowBinaryException(XServerError,UnableToLoadFont,draw_info->font);
     }
   (void) LogMagickEvent(AnnotateEvent,GetMagickModule(),
     "Font %.1024s; pointsize %g",draw_info->font != (char *) NULL ?
@@ -1740,9 +1771,9 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
   metrics->underline_position=(-2.0);
   metrics->underline_thickness=1.0;
   if (!draw_info->render)
-    return(True);
+    return(MagickPass);
   if (draw_info->fill.opacity == TransparentOpacity)
-    return(True);
+    return(MagickPass);
   /*
     Render fill color.
   */
@@ -1761,17 +1792,17 @@ static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
   pixel.pen_color.red=ScaleQuantumToShort(draw_info->fill.red);
   pixel.pen_color.green=ScaleQuantumToShort(draw_info->fill.green);
   pixel.pen_color.blue=ScaleQuantumToShort(draw_info->fill.blue);
-  status=XAnnotateImage(display,&pixel,&annotate_info,image);
+  status=MagickXAnnotateImage(display,&pixel,&annotate_info,image);
   if (status == 0)
-    ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-      "UnableToAnnotateImage");
-  return(True);
+    ThrowBinaryException3(ResourceLimitError,MemoryAllocationFailed,
+      UnableToAnnotateImage);
+  return(MagickPass);
 }
 #else
-static unsigned int RenderX11(Image *image,const DrawInfo *draw_info,
-  const PointInfo *offset,TypeMetric *metrics)
+static MagickPassFail RenderX11(Image *image,const DrawInfo *draw_info,
+  const PointInfo *ARGUNUSED(offset),TypeMetric *ARGUNUSED(metrics))
 {
-  ThrowBinaryException(MissingDelegateError,"XWindowLibraryIsNotAvailable",
+  ThrowBinaryException(MissingDelegateError,XWindowLibraryIsNotAvailable,
     draw_info->font);
 }
 #endif
