@@ -185,7 +185,7 @@ PixelIterateMonoRead(PixelIteratorMonoReadCallback call_back,
   (void) SetRegionThreads(max_threads,options,columns,rows);
 
 #if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(static,16) shared(row_count, status)
+#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
 #endif
   for (row=y; row < (long) (y+rows); row++)
     {
@@ -316,7 +316,7 @@ PixelIterateMonoModify(PixelIteratorMonoModifyCallback call_back,
   (void) SetRegionThreads(max_threads,options,columns,rows);
 
 #if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(static,16) shared(row_count, status)
+#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
 #endif
   for (row=y; row < (long) (y+rows); row++)
     {
@@ -463,7 +463,7 @@ PixelIterateDualRead(PixelIteratorDualReadCallback call_back,
   (void) SetRegionThreads(max_threads,options,columns,rows);
 
 #if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(static,16) shared(row_count, status)
+#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
 #endif
   for (row=0; row < (long) rows; row++)
     {
@@ -627,7 +627,7 @@ PixelIterateDualImplementation(PixelIteratorDualModifyCallback call_back,
   (void) SetRegionThreads(max_threads,options,columns,rows);
 
 #if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(static,16) shared(row_count, status)
+#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
 #endif
   for (row=0; row < (long) rows; row++)
     {
@@ -923,7 +923,7 @@ PixelIterateTripleImplementation(PixelIteratorTripleModifyCallback call_back,
   (void) SetRegionThreads(max_threads,options,columns,rows);
 
 #if defined(HAVE_OPENMP)
-#  pragma omp parallel for schedule(static,16) shared(row_count, status)
+#  pragma omp parallel for schedule(dynamic,4) shared(row_count, status)
 #endif
   for (row=0; row < (long) rows; row++)
     {
